"""Spec constants for MLX-Knife 2.0 JSON API.

Single source of truth for the JSON API specification version used by the
current code and tests. Keep this in sync with docs/json-api-specification.md.
"""

JSON_API_SPEC_VERSION = "0.1.6"
